@@ -1,22 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2011 Belledonne Communications SARL
-  Author: Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #include "ortp/ortp.h"
 #include "utils.h"
@@ -35,10 +34,10 @@ static OrtpNetworkSimulatorCtx* simulator_ctx_new(void){
 	return ctx;
 }
 
-void ortp_network_simulator_destroy(OrtpNetworkSimulatorCtx *sim){
+static void ortp_network_simulator_dump_stats(OrtpNetworkSimulatorCtx *sim) {
 	int drop_by_flush=sim->latency_q.q_mcount+sim->q.q_mcount;
 	if (sim->total_count>0){
-		ortp_message("Network simulation: destroyed. Statistics are:"
+		ortp_message("Network simulation: dump stats. Statistics are:"
 			"%d/%d(%.1f%%, param=%.1f) packets dropped by loss, "
 			"%d/%d(%.1f%%) packets dropped by congestion, "
 			"%d/%d(%.1f%%) packets flushed."
@@ -47,6 +46,9 @@ void ortp_network_simulator_destroy(OrtpNetworkSimulatorCtx *sim){
 			, drop_by_flush, sim->total_count, drop_by_flush*100.f/sim->total_count
 		);
 	}
+}
+void ortp_network_simulator_destroy(OrtpNetworkSimulatorCtx *sim){
+	ortp_network_simulator_dump_stats(sim);
 	flushq(&sim->latency_q,0);
 	flushq(&sim->q,0);
 	flushq(&sim->send_q,0);
@@ -144,8 +146,11 @@ OrtpNetworkSimulatorMode ortp_network_simulator_mode_from_string(const char *str
 void rtp_session_enable_network_simulation(RtpSession *session, const OrtpNetworkSimulatorParams *params){
 	OrtpNetworkSimulatorCtx *sim=session->net_sim_ctx;
 	if (params->enabled){
-		if (sim==NULL)
+		if (sim==NULL) {
 			sim=simulator_ctx_new();
+		} else {
+			ortp_network_simulator_dump_stats(sim);
+		}
 		sim->drop_by_congestion=sim->drop_by_loss=sim->total_count=0;
 		sim->params=*params;
 		if (sim->params.jitter_burst_density>0 && sim->params.jitter_strength>0 && sim->params.max_bandwidth==0){
@@ -171,7 +176,7 @@ void rtp_session_enable_network_simulation(RtpSession *session, const OrtpNetwor
 				"\tmax_buffer_size=%d\n"
 				"\tjitter_density=%.1f\n"
 				"\tjitter_strength=%.1f\n"
-				"\tmode=%s\n",
+				"\tmode=%s",
 				params->latency,
 				params->loss_rate,
 				params->consecutive_loss_probability,
@@ -311,7 +316,7 @@ static mblk_t *simulate_loss_rate(OrtpNetworkSimulatorCtx *net_sim_ctx, mblk_t *
 	int rrate;
 	float loss_rate=net_sim_ctx->params.loss_rate*10.0f;
 
-	/*in order to simulate bursts of dropped packets, take into account a different probability after a loss occured*/
+	/*in order to simulate bursts of dropped packets, take into account a different probability after a loss occurred*/
 	if (net_sim_ctx->consecutive_drops>0){
 		loss_rate=net_sim_ctx->params.consecutive_loss_probability*1000.0f;
 	}
@@ -355,7 +360,7 @@ mblk_t * rtp_session_network_simulate(RtpSession *session, mblk_t *input, bool_t
 	}
 
 	if ((sim->params.loss_rate > 0) && (om != NULL)) {
-		if (sim->params.RTP_only == TRUE) {
+		if (sim->params.rtp_only == TRUE) {
 			if (*is_rtp_packet == TRUE) {
 				om = simulate_loss_rate(sim, om);
 			}
@@ -394,7 +399,6 @@ int ortp_timespec_compare(const ortpTimeSpec *s1, const ortpTimeSpec *s2){
 		return 1;
 }
 
-#if defined(ORTP_TIMESTAMP)
 static mblk_t * rtp_session_netsim_find_next_packet_to_send(RtpSession *session){
 	mblk_t *om;
 	ortpTimeSpec min_packet_time = { 0, 0};
@@ -415,7 +419,6 @@ static mblk_t * rtp_session_netsim_find_next_packet_to_send(RtpSession *session)
 	}
 	return next_packet;
 }
-#endif
 
 static void rtp_session_schedule_outbound_network_simulator(RtpSession *session, ortpTimeSpec *sleep_until){
 	mblk_t *om;
@@ -454,13 +457,13 @@ static void rtp_session_schedule_outbound_network_simulator(RtpSession *session,
 			}
 		}
 	}else if (session->net_sim_ctx->params.mode==OrtpNetworkSimulatorOutboundControlled){
-#if defined(ORTP_TIMESTAMP)
 		ortpTimeSpec current={0};
 		ortpTimeSpec packet_time;
 		mblk_t *todrop=NULL;
 
 		ortp_mutex_lock(&session->net_sim_ctx->mutex);
 		while((om = rtp_session_netsim_find_next_packet_to_send(session)) != NULL){
+			is_rtp_packet=om->reserved1; /*it was set by rtp_session_sendto()*/
 			ortp_mutex_unlock(&session->net_sim_ctx->mutex);
 			if (todrop) {
 				freemsg(todrop); /*free the last message while the mutex is not held*/
@@ -469,11 +472,11 @@ static void rtp_session_schedule_outbound_network_simulator(RtpSession *session,
 			_ortp_get_cur_time(&current,TRUE);
 			packet_time.tv_sec=om->timestamp.tv_sec;
 			packet_time.tv_nsec=om->timestamp.tv_usec*1000LL;
-			if (om->timestamp.tv_sec==0 && om->timestamp.tv_usec==0){
-				todrop = om; /*simulate a packet loss*/
+			if (is_rtp_packet && om->timestamp.tv_sec==0 && om->timestamp.tv_usec==0){
+				todrop = om; /*simulate a packet loss, only RTP packets can be dropped. Timestamp is not set for RTCP packets*/
 			}else if (ortp_timespec_compare(&packet_time, &current) <= 0){
 				/*it is time to send this packet*/
-				is_rtp_packet=om->reserved1; /*it was set by rtp_session_sendto()*/
+				
 				_ortp_sendto(is_rtp_packet ? session->rtp.gs.socket : session->rtcp.gs.socket, om, 0, (struct sockaddr*)&om->net_addr, om->net_addrlen);
 				todrop = om;
 			}else {
@@ -493,16 +496,6 @@ static void rtp_session_schedule_outbound_network_simulator(RtpSession *session,
 			sleep_until->tv_sec=current.tv_sec;
 			sleep_until->tv_nsec=current.tv_nsec+1000000LL; /*in 1 ms*/
 		}
-#else
-		ortp_mutex_lock(&session->net_sim_ctx->mutex);
-		while((om=getq(&session->net_sim_ctx->send_q))!=NULL){
-			ortp_mutex_unlock(&session->net_sim_ctx->mutex);
-			freemsg(om);
-			ortp_error("Network simulator is in mode OrtpNetworkSimulatorOutboundControlled but oRTP wasn't compiled with --enable-ntp-timestamp.");
-			ortp_mutex_lock(&session->net_sim_ctx->mutex);
-		}
-		ortp_mutex_unlock(&session->net_sim_ctx->mutex);
-#endif
 	}
 }
 
