@@ -1,22 +1,21 @@
-
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -26,6 +25,7 @@
 #include "ortp/port.h"
 #include "ortp/str_utils.h"
 #include "utils.h"
+#include <bctoolbox/port.h>
 
 #if	defined(_WIN32) && !defined(_WIN32_WCE)
 #include <process.h>
@@ -713,7 +713,7 @@ void ortp_shm_close(void *mem){
 #endif
 
 void _ortp_get_cur_time(ortpTimeSpec *ret, bool_t realtime){
-#if defined(_WIN32_WCE) || defined(WIN32)
+#if defined(_WIN32_WCE) || defined(_WIN32)
 #ifdef ORTP_WINDOWS_DESKTOP
 	DWORD timemillis;
 #	if defined(_WIN32_WCE)
@@ -832,12 +832,14 @@ char* strtok_r(char *str, const char *delim, char **nextp){
 #endif
 
 
-#if defined(_WIN32) && !defined(_MSC_VER)
+#if defined(_WIN32)
+
+#if !defined(_MSC_VER)
 #include <wincrypt.h>
 static int ortp_wincrypto_random(unsigned int *rand_number){
 	static HCRYPTPROV hProv=(HCRYPTPROV)-1;
 	static int initd=0;
-	
+
 	if (!initd){
 		if (!CryptAcquireContext(&hProv,NULL,NULL,PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)){
 			ortp_error("ortp_wincrypto_random(): Could not acquire a windows crypto context");
@@ -847,7 +849,7 @@ static int ortp_wincrypto_random(unsigned int *rand_number){
 	}
 	if (hProv==(HCRYPTPROV)-1)
 		return -1;
-	
+
 	if (!CryptGenRandom(hProv,4,(BYTE*)rand_number)){
 		ortp_error("ortp_wincrypto_random(): CryptGenRandom() failed.");
 		return -1;
@@ -855,11 +857,9 @@ static int ortp_wincrypto_random(unsigned int *rand_number){
 	return 0;
 }
 #endif
+#elif defined(__QNXNTO__) || ((defined(__linux) || defined(__APPLE__)) && !defined(HAVE_ARC4RANDOM))
 
-unsigned int ortp_random(void){
-#ifdef HAVE_ARC4RANDOM
-	return arc4random();
-#elif  defined(__linux) || defined(__APPLE__)
+static unsigned int ortp_urandom(void) {
 	static int fd=-1;
 	if (fd==-1) fd=open("/dev/urandom",O_RDONLY);
 	if (fd!=-1){
@@ -867,13 +867,30 @@ unsigned int ortp_random(void){
 		if (read(fd,&tmp,4)!=4){
 			ortp_error("Reading /dev/urandom failed.");
 		}else return tmp;
-	}else ortp_error("Could not open /dev/urandom");
+	} else ortp_error("Could not open /dev/urandom");
+	return (unsigned int) random();
+}
+
+#endif
+
+
+
+
+unsigned int ortp_random(void){
+#ifdef HAVE_ARC4RANDOM
+#if defined(__QNXNTO__) // There is a false positive with blackberry build
+	return ortp_urandom();
+#else
+	return arc4random();
+#endif
+#elif defined(__linux) || defined(__APPLE__)
+	return ortp_urandom();
 #elif defined(_WIN32)
 	static int initd=0;
 	unsigned int ret;
 #ifdef _MSC_VER
 	/*rand_s() is pretty nice and simple function but is not wrapped by mingw.*/
-	
+
 	if (rand_s(&ret)==0){
 		return ret;
 	}
@@ -891,10 +908,18 @@ unsigned int ortp_random(void){
 		ortp_warning("ortp: Random generator is using rand(), this is unsecure !");
 	}
 	return rand()<<16 | rand();
-#endif
-	/*fallback to UNIX random()*/
-#ifndef _WIN32
-	return (unsigned int) random();
 #endif
 }
 
+bool_t ortp_is_multicast_addr(const struct sockaddr *addr) {
+
+	switch (addr->sa_family) {
+		case AF_INET:
+			return IN_MULTICAST(ntohl(((struct sockaddr_in *) addr)->sin_addr.s_addr));
+		case AF_INET6:
+			return IN6_IS_ADDR_MULTICAST(&(((struct sockaddr_in6 *) addr)->sin6_addr));
+		default:
+			return FALSE;
+	}
+
+}
