@@ -1,21 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 /***************************************************************************
  *            rtcp.c
@@ -234,7 +234,7 @@ static void report_block_init(report_block_t *b, RtpSession *session){
 
 	/* compute the statistics */
 	if (stream->hwrcv_since_last_SR!=0){
-		int expected_packets=(int)((int64_t)stream->hwrcv_extseq - (int64_t)stream->hwrcv_seq_at_last_SR);
+		uint32_t expected_packets=(uint32_t)(stream->hwrcv_extseq - stream->hwrcv_seq_at_last_SR);
 
 		if ( session->flags & RTCP_OVERRIDE_LOST_PACKETS ) {
 			/* If the test mode is enabled, replace the lost packet field with
@@ -245,7 +245,7 @@ static void report_block_init(report_block_t *b, RtpSession *session){
 			session->stats.cum_packet_loss = packet_loss;
 		}else {
 			/* Normal mode */
-			packet_loss = (int)((int64_t)expected_packets - (int64_t)stream->hwrcv_since_last_SR);
+			packet_loss = (int)(expected_packets - stream->hwrcv_since_last_SR);
 			session->stats.cum_packet_loss += packet_loss;
 		}
 		if (expected_packets>0){/*prevent division by zero and negative loss fraction*/
@@ -338,10 +338,10 @@ static void extended_statistics( RtpSession *session, report_block_t * rb ) {
 	session->rtp.jitter_stats.jitter_buffer_size_ms=jitter_control_compute_mean_size(&session->rtp.jittctl);
 }
 
-static int rtcp_sr_init(RtpSession *session, uint8_t *buf, int size){
+static size_t rtcp_sr_init(RtpSession *session, uint8_t *buf, size_t size){
 	rtcp_sr_t *sr=(rtcp_sr_t*)buf;
 	int rr=(session->stats.packet_recv>0);
-	int sr_size=sizeof(rtcp_sr_t)-sizeof(report_block_t)+(rr*sizeof(report_block_t));
+	size_t sr_size=sizeof(rtcp_sr_t)-sizeof(report_block_t)+(rr*sizeof(report_block_t));
 	if (size<sr_size) return 0;
 	rtcp_common_header_init(&sr->ch,session,RTCP_SR,rr,sr_size);
 	sr->ssrc=htonl(session->snd.ssrc);
@@ -354,7 +354,7 @@ static int rtcp_sr_init(RtpSession *session, uint8_t *buf, int size){
 	return sr_size;
 }
 
-static int rtcp_rr_init(RtpSession *session, uint8_t *buf, int size){
+static size_t rtcp_rr_init(RtpSession *session, uint8_t *buf, size_t size){
 	rtcp_rr_t *rr=(rtcp_rr_t*)buf;
 	if (size<sizeof(rtcp_rr_t)) return 0;
 	rtcp_common_header_init(&rr->ch,session,RTCP_RR,1,sizeof(rtcp_rr_t));
@@ -364,7 +364,7 @@ static int rtcp_rr_init(RtpSession *session, uint8_t *buf, int size){
 	return sizeof(rtcp_rr_t);
 }
 
-static int rtcp_app_init(RtpSession *session, uint8_t *buf, uint8_t subtype, const char *name, int size){
+static size_t rtcp_app_init(RtpSession *session, uint8_t *buf, uint8_t subtype, const char *name, size_t size){
 	rtcp_app_t *app=(rtcp_app_t*)buf;
 	if (size<sizeof(rtcp_app_t)) return 0;
 	rtcp_common_header_init(&app->ch,session,RTCP_APP,subtype,size);
@@ -687,8 +687,8 @@ void ortp_loss_rate_estimator_init(OrtpLossRateEstimator *obj, int min_packet_co
 bool_t ortp_loss_rate_estimator_process_report_block(OrtpLossRateEstimator *obj, const RtpSession *session, const report_block_t *rb){
 	int32_t cum_loss=report_block_get_cum_packet_lost(rb);
 	int32_t extseq=report_block_get_high_ext_seq(rb);
-	int32_t diff_unique_outgoing=(int32_t)(session->stats.packet_sent-obj->last_packet_sent_count);
-	int32_t diff_total_outgoing=diff_unique_outgoing+(int32_t)(session->stats.packet_dup_sent-obj->last_dup_packet_sent_count);
+	//int32_t diff_unique_outgoing=(int32_t)(session->stats.packet_sent-obj->last_packet_sent_count);
+	//int32_t diff_total_outgoing=diff_unique_outgoing+(int32_t)(session->stats.packet_dup_sent-obj->last_dup_packet_sent_count);
 	int32_t diff;
 	uint64_t curtime;
 	bool_t got_value=FALSE;
@@ -703,7 +703,14 @@ bool_t ortp_loss_rate_estimator_process_report_block(OrtpLossRateEstimator *obj,
 	diff=extseq-obj->last_ext_seq;
 	curtime=ortp_get_cur_time_ms();
 	if (diff<0 || diff>obj->min_packet_count_interval * 100){
-		ortp_warning("ortp_loss_rate_estimator_process %p: Suspected discontinuity in sequence numbering from %d to %d.", obj, obj->last_ext_seq, extseq);
+		if (extseq==0){
+			/*when extseq reset to 0, it probably means that rtp_session_sync was called but
+			since OrtplossRateEstimator is not reset, first RTCP packet received will be detected
+			as discontinuity instead of init RTCP packet. Avoid logging in such case.*/
+			ortp_message("ortp_loss_rate_estimator_process %p: Suspected RTP session restart, sequence numbering from %d to %d.", obj, obj->last_ext_seq, extseq);
+		}else{
+			ortp_warning("ortp_loss_rate_estimator_process %p: Suspected discontinuity in sequence numbering from %d to %d.", obj, obj->last_ext_seq, extseq);
+		}
 		obj->last_ext_seq=extseq;
 		obj->last_cum_loss=cum_loss;
 		obj->last_packet_sent_count=session->stats.packet_sent;
@@ -711,17 +718,28 @@ bool_t ortp_loss_rate_estimator_process_report_block(OrtpLossRateEstimator *obj,
 	}else if (diff>obj->min_packet_count_interval && curtime-obj->last_estimate_time_ms>=obj->min_time_ms_interval){
 		/*we have sufficient interval*/
 		int32_t new_losses=cum_loss-obj->last_cum_loss;
+		
+#if 0 /*SM: the following code try to takes into account sent duplicates - however by doing this it creates a bias in the loss rate computation
+		that can sometimes result in a negative loss rate, even if there is no duplicate.
+		Since the rate control doesn't use duplicates anymore, there is no good reason to take this into account.
+		*/
 		/*if we are using duplicates, they will not be visible in 'diff' variable.
 		But since we are the emitter, we can retrieve the total count of packet we
 		sent and use this value to compute the loss rate instead.*/
 		obj->loss_rate = 100.f * (1.f - MAX(0, (diff_unique_outgoing - new_losses) * 1.f / diff_total_outgoing));
+#endif
+		obj->loss_rate = 100.f * (float) new_losses / (float)( extseq - obj->last_ext_seq);
 
 		/*update last values with current*/
 		got_value=TRUE;
 		obj->last_estimate_time_ms=curtime;
 
 		if (obj->loss_rate>100.f){
+			obj->loss_rate = 100.f;
 			ortp_error("ortp_loss_rate_estimator_process %p: Loss rate MUST NOT be greater than 100%%", obj);
+		}else if(obj->loss_rate < 0){
+			obj->loss_rate = 0;
+			ortp_error("ortp_loss_rate_estimator_process %p: Loss rate MUST NOT be negative", obj);
 		}
 		obj->last_ext_seq=extseq;
 		obj->last_cum_loss=cum_loss;
@@ -736,9 +754,6 @@ float ortp_loss_rate_estimator_get_value(OrtpLossRateEstimator *obj){
 	return obj->loss_rate;
 }
 
-void ortp_loss_rate_estimator_uninit(OrtpLossRateEstimator *obj){
-}
-
 void ortp_loss_rate_estimator_destroy(OrtpLossRateEstimator *obj){
 	ortp_free(obj);
 }
