@@ -1,21 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 
 #include "ortp/ortp.h"
@@ -27,11 +27,11 @@ static size_t rtcp_get_size(const mblk_t *m){
 	return (1+rtcp_common_header_get_length(ch))*4;
 }
 
-/*in case of coumpound packet, set read pointer of m to the beginning of the next RTCP
+/*in case of compound packet, set read pointer of m to the beginning of the next RTCP
 packet */
 bool_t rtcp_next_packet(mblk_t *m){
 	size_t nextlen=rtcp_get_size(m);
-	if (m->b_rptr+nextlen<m->b_wptr){
+	if ((nextlen > 0) && (m->b_rptr + nextlen < m->b_wptr)){
 		m->b_rptr+=nextlen;
 		return TRUE;
 	}
@@ -51,7 +51,7 @@ const rtcp_common_header_t * rtcp_get_common_header(const mblk_t *m){
 		return NULL;
 	}
 	if (size<sizeof(rtcp_common_header_t)){
-		ortp_warning("Bad RTCP packet, too short.");
+		ortp_warning("Bad RTCP packet, too short [%i b]. on block [%p]",(int)size,m);
 		return NULL;
 	}
 	ch=(rtcp_common_header_t*)m->b_rptr;
@@ -206,10 +206,9 @@ bool_t rtcp_is_BYE(const mblk_t *m){
 bool_t rtcp_BYE_get_ssrc(const mblk_t *m, int idx, uint32_t *ssrc){
 	rtcp_bye_t *bye=(rtcp_bye_t*)m->b_rptr;
 	int rc=rtcp_common_header_get_rc(&bye->ch);
-	int len=rtcp_common_header_get_length(&bye->ch);
 	if (idx<rc){
 		if ((uint8_t*)&bye->ssrc[idx]<=(m->b_rptr
-				+sizeof(rtcp_common_header_t)+len-4)) {
+				+ rtcp_get_size(m)-4)) {
 			*ssrc=ntohl(bye->ssrc[idx]);
 			return TRUE;
 		}else{
@@ -558,6 +557,11 @@ rtcp_fb_tmmbr_fci_t * rtcp_RTPFB_tmmbr_get_fci(const mblk_t *m) {
 	return (rtcp_fb_tmmbr_fci_t *)(m->b_rptr + size - sizeof(rtcp_fb_tmmbr_fci_t));
 }
 
+uint64_t rtcp_RTPFB_tmmbr_get_max_bitrate(const mblk_t *m) {
+	rtcp_fb_tmmbr_fci_t *fci = rtcp_RTPFB_tmmbr_get_fci(m);
+	return rtcp_fb_tmmbr_fci_get_mxtbr_mantissa(fci) * (1 << rtcp_fb_tmmbr_fci_get_mxtbr_exp(fci));
+}
+
 
 /* RTCP PSFB accessors */
 bool_t rtcp_is_PSFB(const mblk_t *m) {
