@@ -1,21 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -26,6 +26,8 @@
 #include "jitterctl.h"
 #include "utils.h"
 #include "rtpsession_priv.h"
+#include "congestiondetector.h"
+#include "videobandwidthestimator.h"
 
 static bool_t queue_packet(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int *discarded, int *duplicate)
 {
@@ -130,13 +132,13 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	rtp_header_t *rtp;
 	int msgsize;
 	RtpStream *rtpstream=&session->rtp;
-	rtp_stats_t *stats=&rtpstream->stats;
+	rtp_stats_t *stats=&session->stats;
 
 	msgsize=(int)(mp->b_wptr-mp->b_rptr);
 
 	if (msgsize<RTP_FIXED_HEADER_SIZE){
 		ortp_warning("Packet too small to be a rtp packet (%i)!",msgsize);
-		rtpstream->stats.bad++;
+		session->stats.bad++;
 		ortp_global_stats.bad++;
 		freemsg(mp);
 		return;
@@ -149,6 +151,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		stunlen = ntohs(stunlen);
 		if (stunlen+20==mp->b_wptr-mp->b_rptr){
 			/* this looks like a stun packet */
+			rtp_session_update_remote_sock_addr(session,mp,TRUE,TRUE);
 			if (session->eventqs!=NULL){
 				OrtpEvent *ev=ortp_event_new(ORTP_EVENT_STUN_PACKET_RECEIVED);
 				OrtpEventData *ed=ortp_event_get_data(ev);
@@ -161,7 +164,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			}
 		}
 		/* discard in two case: the packet is not stun OR nobody is interested by STUN (no eventqs) */
-		ortp_debug("Receiving rtp packet with version number !=2...discarded");
+		ortp_debug("Receiving rtp packet with version number %d!=2...discarded", rtp->version);
 		stats->bad++;
 		ortp_global_stats.bad++;
 		freemsg(mp);
@@ -209,13 +212,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			}
 			if (session->inc_same_ssrc_count>=session->rtp.ssrc_changed_thres){
 				/* store the sender rtp address to do symmetric RTP */
-				if (!session->use_connect){
-					if (session->rtp.gs.socket>0 && session->symmetric_rtp){
-						/* store the sender rtp address to do symmetric RTP */
-						memcpy(&session->rtp.gs.rem_addr,addr,addrlen);
-						session->rtp.gs.rem_addrlen=addrlen;
-					}
-				}
+				rtp_session_update_remote_sock_addr(session,mp,TRUE,FALSE);
 				session->rtp.rcv_last_ts = rtp->timestamp;
 				session->rcv.ssrc=rtp->ssrc;
 				rtp_signal_table_emit(&session->on_ssrc_changed);
@@ -235,14 +232,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	}else{
 		session->ssrc_set=TRUE;
 		session->rcv.ssrc=rtp->ssrc;
-
-		if (!session->use_connect){
-			if (session->rtp.gs.socket>0 && session->symmetric_rtp){
-				/* store the sender rtp address to do symmetric RTP */
-				memcpy(&session->rtp.gs.rem_addr,addr,addrlen);
-				session->rtp.gs.rem_addrlen=addrlen;
-			}
-		}
+		rtp_session_update_remote_sock_addr(session,mp,TRUE,FALSE);
 	}
 
 	/* update some statistics */
@@ -271,7 +261,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 
 	/* check for possible telephone events */
 	if (rtp_profile_is_telephone_event(session->snd.profile, rtp->paytype)){
-		queue_packet(&session->rtp.tev_rq,session->rtp.max_rq_size,mp,rtp,&discarded,&duplicate);
+		queue_packet(&session->rtp.tev_rq,session->rtp.jittctl.params.max_packets,mp,rtp,&discarded,&duplicate);
 		stats->discarded+=discarded;
 		ortp_global_stats.discarded+=discarded;
 		stats->packet_dup_recv+=duplicate;
@@ -295,6 +285,20 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 
 	jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts);
 
+	if (session->video_bandwidth_estimator_enabled && session->rtp.video_bw_estimator) {
+		int overhead = ortp_stream_is_ipv6(&session->rtp.gs) ? IP6_UDP_OVERHEAD : IP_UDP_OVERHEAD;
+		ortp_video_bandwidth_estimator_process_packet(session->rtp.video_bw_estimator, rtp->timestamp, &mp->timestamp, msgsize + overhead, rtp->markbit == 1);
+	}
+
+	if (session->congestion_detector_enabled && session->rtp.congdetect){
+		if (ortp_congestion_detector_record(session->rtp.congdetect,rtp->timestamp,local_str_ts)) {
+			OrtpEvent *ev=ortp_event_new(ORTP_EVENT_CONGESTION_STATE_CHANGED);
+			OrtpEventData *ed=ortp_event_get_data(ev);
+			ed->info.congestion_detected = session->rtp.congdetect->state == CongestionStateDetected;
+			rtp_session_dispatch_event(session,ev);
+		}
+	}
+	
 	update_rtcp_xr_stat_summary(session, mp, local_str_ts);
 
 	if (session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED) {
@@ -303,7 +307,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			ortp_warning("rtp_parse: timestamp jump in the future detected.");
 			rtp_signal_table_emit2(&session->on_timestamp_jump,&rtp->timestamp);
 		}
-		else if (RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN(session->rtp.rcv_last_ts,rtp->timestamp) 
+		else if (RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN(session->rtp.rcv_last_ts,rtp->timestamp)
 			|| RTP_SEQ_IS_STRICTLY_GREATER_THAN(session->rtp.rcv_last_seq,rtp->seq_number)){
 			/* don't queue packets older than the last returned packet to the application, or whose sequence number
 			 is behind the last packet returned to the application*/
@@ -315,7 +319,8 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 				ortp_warning("rtp_parse: negative timestamp jump detected");
 				rtp_signal_table_emit2(&session->on_timestamp_jump, &rtp->timestamp);
 			}
-			ortp_debug("rtp_parse: discarding too old packet (ts=%i)",rtp->timestamp);
+			ortp_error("rtp_parse: discarding too old packet (seq=%i, ts=%u, last_delivered was seq=%i, ts=%u)",rtp->seq_number, rtp->timestamp,
+				(int)session->rtp.rcv_last_seq, session->rtp.rcv_last_ts);
 			freemsg(mp);
 			stats->outoftime++;
 			ortp_global_stats.outoftime++;
@@ -324,7 +329,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		}
 	}
 
-	if (queue_packet(&session->rtp.rq,session->rtp.max_rq_size,mp,rtp,&discarded,&duplicate))
+	if (queue_packet(&session->rtp.rq,session->rtp.jittctl.params.max_packets,mp,rtp,&discarded,&duplicate))
 		jitter_control_update_size(&session->rtp.jittctl,&session->rtp.rq);
 	stats->discarded+=discarded;
 	ortp_global_stats.discarded+=discarded;
