@@ -1,27 +1,31 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
 
+#if __APPLE__
+#include "TargetConditionals.h"
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include "ortp-config.h" /*needed for HAVE_SYS_UIO_H and HAVE_ARC4RANDOM */
 #endif
@@ -205,7 +209,7 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 			}
 		}
 
-#if defined(ORTP_TIMESTAMP)
+#ifdef SO_TIMESTAMP
 		optval=1;
 		err = setsockopt (sock, SOL_SOCKET, SO_TIMESTAMP,
 			(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
@@ -220,12 +224,12 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 			default:
 			case AF_INET:
 #ifdef IP_RECVTTL
-				err = setsockopt(sock, IPPROTO_IP, IP_RECVTTL, &optval, sizeof(optval));
+				err = setsockopt(sock, IPPROTO_IP, IP_RECVTTL, (SOCKET_OPTION_VALUE)&optval, sizeof(optval));
 #endif
 				break;
 			case AF_INET6:
 #ifdef IPV6_RECVHOPLIMIT
-				err = setsockopt(sock, IPPROTO_IPV6, IPV6_RECVHOPLIMIT, &optval, sizeof(optval));
+				err = setsockopt(sock, IPPROTO_IPV6, IPV6_RECVHOPLIMIT, (SOCKET_OPTION_VALUE)&optval, sizeof(optval));
 #endif
 				break;
 		}
@@ -368,7 +372,6 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int rtp_por
 		}
 
 		/* set socket options (but don't change chosen states) */
-		rtp_session_set_dscp( session, -1 );
 		rtp_session_set_multicast_ttl( session, -1 );
 		rtp_session_set_multicast_loopback( session, -1 );
 		if (session->use_pktinfo) rtp_session_set_pktinfo(session, TRUE);
@@ -446,41 +449,38 @@ int rtp_session_set_pktinfo(RtpSession *session, int activate)
 	memset(optval, activate, sizeof(optval));
 #endif
 
-	switch (session->rtp.gs.sockfamily) {
-		case AF_INET:
 #ifdef IP_PKTINFO
-			optname = IP_PKTINFO;
+	optname = IP_PKTINFO;
 #else
-			optname = IP_RECVDSTADDR;
+	optname = IP_RECVDSTADDR;
 #endif
-			retval = setsockopt(session->rtp.gs.socket, IPPROTO_IP, optname, optval, optlen);
-			if (retval < 0) {
-				ortp_warning ("Fail to set packet info on RTP socket: %s.", getSocketError());
-			}
-			retval = setsockopt(session->rtcp.gs.socket, IPPROTO_IP, optname, optval, optlen);
-			if (retval < 0) {
-				ortp_warning ("Fail to set packet info on RTCP socket: %s.", getSocketError());
-			}
-			break;
-		case AF_INET6:
+	retval = setsockopt(session->rtp.gs.socket, IPPROTO_IP, optname, optval, optlen);
+	if (retval < 0) {
+		ortp_warning ("Fail to set IPv4 packet info on RTP socket: %s.", getSocketError());
+	}
+	retval = setsockopt(session->rtcp.gs.socket, IPPROTO_IP, optname, optval, optlen);
+	if (retval < 0) {
+		ortp_warning ("Fail to set IPv4 packet info on RTCP socket: %s.", getSocketError());
+	}
+
+	if (session->rtp.gs.sockfamily != AF_INET) {
+#if defined(_WIN32) || defined(_WIN32_WCE)
+		memset(optval, activate, sizeof(optval));
+#endif
+
 #ifdef IPV6_RECVPKTINFO
-			optname = IPV6_RECVPKTINFO;
+		optname = IPV6_RECVPKTINFO;
 #else
-			optname = IPV6_RECVDSTADDR;
+		optname = IPV6_RECVDSTADDR;
 #endif
-			retval = setsockopt(session->rtp.gs.socket, IPPROTO_IPV6, optname, optval, optlen);
-			if (retval < 0) {
-				ortp_warning ("Fail to set packet info on RTP socket: %s.", getSocketError());
-			}
-			retval = setsockopt(session->rtcp.gs.socket, IPPROTO_IPV6, optname, optval, optlen);
-			if (retval < 0) {
-				ortp_warning ("Fail to set packet info on RTCP socket: %s.", getSocketError());
-			}
-			break;
-		default:
-			retval = -1;
-			ortp_warning("Unknow socket family %d used to set packet info", session->rtp.gs.sockfamily);
-			break;
+		retval = setsockopt(session->rtp.gs.socket, IPPROTO_IPV6, optname, optval, optlen);
+		if (retval < 0) {
+			ortp_warning("Fail to set IPv6 packet info on RTP socket: %s.", getSocketError());
+		}
+		retval = setsockopt(session->rtcp.gs.socket, IPPROTO_IPV6, optname, optval, optlen);
+		if (retval < 0) {
+			ortp_warning("Fail to set IPv6 packet info on RTCP socket: %s.", getSocketError());
+		}
 	}
 
 	return retval;
@@ -556,9 +556,9 @@ int rtp_session_get_multicast_ttl(RtpSession *session)
 
 /**
  *@param session: a rtp session
- *@param yesno: desired Multicast Time-To-Live
+ *@param yesno: enable multicast loopback
  *
- * Sets the TTL (Time-To-Live) for outgoing multicast packets.
+ * Enable multicast loopback.
  *
  * Returns: 0 on success.
  *
@@ -759,8 +759,8 @@ int rtp_session_get_local_rtcp_port(const RtpSession *session){
 /**
  *rtp_session_set_remote_addr:
  *@param session:		a rtp session freshly created.
- *@param addr:		a local IP address in the xxx.xxx.xxx.xxx form.
- *@param port:		a local port.
+ *@param addr:		a remote IP address in the xxx.xxx.xxx.xxx form.
+ *@param port:		a remote port.
  *
  *	Sets the remote address of the rtp session, ie the destination address where rtp packet
  *	are sent. If the session is recv-only or duplex, it also sets the origin of incoming RTP
@@ -776,10 +776,10 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port){
 /**
  *rtp_session_set_remote_addr_full:
  *@param session:		a rtp session freshly created.
- *@param rtp_addr:		a local IP address in the xxx.xxx.xxx.xxx form.
- *@param rtp_port:		a local rtp port.
- *@param rtcp_addr:		a local IP address in the xxx.xxx.xxx.xxx form.
- *@param rtcp_port:		a local rtcp port.
+ *@param rtp_addr:		a remote IP address in the xxx.xxx.xxx.xxx form.
+ *@param rtp_port:		a remote rtp port.
+ *@param rtcp_addr:		a remote IP address in the xxx.xxx.xxx.xxx form.
+ *@param rtcp_port:		a remote rtcp port.
  *
  *	Sets the remote address of the rtp session, ie the destination address where rtp packet
  *	are sent. If the session is recv-only or duplex, it also sets the origin of incoming RTP
@@ -816,6 +816,7 @@ _rtp_session_set_remote_addr_full (RtpSession * session, const char * rtp_addr,
 
 	res0 = bctbx_name_to_addrinfo((session->rtp.gs.socket == -1) ? AF_UNSPEC : session->rtp.gs.sockfamily, SOCK_DGRAM, rtp_addr, rtp_port);
 	if (res0 == NULL) {
+		ortp_error("_rtp_session_set_remote_addr_full(): cannot set RTP destination to %s port %i.", rtp_addr, rtp_port);
 		err=-1;
 		goto end;
 	} else {
@@ -850,55 +851,60 @@ _rtp_session_set_remote_addr_full (RtpSession * session, const char * rtp_addr,
 		goto end;
 	}
 
-	res0 = bctbx_name_to_addrinfo((session->rtcp.gs.socket == -1) ? AF_UNSPEC : session->rtcp.gs.sockfamily, SOCK_DGRAM, rtcp_addr, rtcp_port);
-	if (res0 == NULL) {
+	if ((rtcp_addr != NULL) && (rtcp_port > 0)) {
+		res0 = bctbx_name_to_addrinfo((session->rtcp.gs.socket == -1) ? AF_UNSPEC : session->rtcp.gs.sockfamily, SOCK_DGRAM, rtcp_addr, rtcp_port);
+		if (res0 == NULL) {
+			ortp_error("_rtp_session_set_remote_addr_full(): cannot set RTCP destination to %s port %i.", rtcp_addr, rtcp_port);
+			err=-1;
+			goto end;
+		} else {
+			bctbx_addrinfo_to_printable_ip_address(res0, rtcp_printable_addr, sizeof(rtcp_printable_addr));
+		}
 		err=-1;
-		goto end;
-	} else {
-		bctbx_addrinfo_to_printable_ip_address(res0, rtcp_printable_addr, sizeof(rtcp_printable_addr));
-	}
-	err=-1;
-	for (res = res0; res; res = res->ai_next) {
-		/* set a destination address that has the same type as the local address */
-		if (res->ai_family==session->rtcp.gs.sockfamily ) {
-			err=0;
-			memcpy(rtcp_saddr, res->ai_addr, res->ai_addrlen);
-			*rtcp_saddr_len=(socklen_t)res->ai_addrlen;
-			break;
+		for (res = res0; res; res = res->ai_next) {
+			/* set a destination address that has the same type as the local address */
+			if (res->ai_family==session->rtcp.gs.sockfamily ) {
+				err=0;
+				memcpy(rtcp_saddr, res->ai_addr, res->ai_addrlen);
+				*rtcp_saddr_len=(socklen_t)res->ai_addrlen;
+				break;
+			}
+		}
+		bctbx_freeaddrinfo(res0);
+		if (err) {
+			ortp_warning("Could not set destination for RCTP socket to %s:%i.",rtcp_addr,rtcp_port);
+			goto end;
 		}
-	}
-	bctbx_freeaddrinfo(res0);
-	if (err) {
-		ortp_warning("Could not set destination for RCTP socket to %s:%i.",rtcp_addr,rtcp_port);
-		goto end;
-	}
 
-	if (can_connect(session)){
-		if (try_connect(session->rtp.gs.socket,(struct sockaddr*)&session->rtp.gs.rem_addr,session->rtp.gs.rem_addrlen))
-			session->flags|=RTP_SOCKET_CONNECTED;
-		if (session->rtcp.gs.socket!=(ortp_socket_t)-1){
-			if (try_connect(session->rtcp.gs.socket,(struct sockaddr*)&session->rtcp.gs.rem_addr,session->rtcp.gs.rem_addrlen))
-				session->flags|=RTCP_SOCKET_CONNECTED;
+		if (can_connect(session)){
+			if (try_connect(session->rtp.gs.socket,(struct sockaddr*)&session->rtp.gs.rem_addr,session->rtp.gs.rem_addrlen))
+				session->flags|=RTP_SOCKET_CONNECTED;
+			if (session->rtcp.gs.socket!=(ortp_socket_t)-1){
+				if (try_connect(session->rtcp.gs.socket,(struct sockaddr*)&session->rtcp.gs.rem_addr,session->rtcp.gs.rem_addrlen))
+					session->flags|=RTCP_SOCKET_CONNECTED;
+			}
+		}else if (session->flags & RTP_SOCKET_CONNECTED){
+			/*must dissolve association done by connect().
+			See connect(2) manpage*/
+			struct sockaddr sa;
+			sa.sa_family=AF_UNSPEC;
+			if (connect(session->rtp.gs.socket,&sa,sizeof(sa))<0){
+				ortp_error("Cannot dissolve connect() association for rtp socket: %s", getSocketError());
+			}
+			if (connect(session->rtcp.gs.socket,&sa,sizeof(sa))<0){
+				ortp_error("Cannot dissolve connect() association for rtcp socket: %s", getSocketError());
+			}
+			session->flags&=~RTP_SOCKET_CONNECTED;
+			session->flags&=~RTCP_SOCKET_CONNECTED;
 		}
-	}else if (session->flags & RTP_SOCKET_CONNECTED){
-		/*must dissolve association done by connect().
-		See connect(2) manpage*/
-		struct sockaddr sa;
-		sa.sa_family=AF_UNSPEC;
-		if (connect(session->rtp.gs.socket,&sa,sizeof(sa))<0){
-			ortp_error("Cannot dissolve connect() association for rtp socket: %s", getSocketError());
-		}
-		if (connect(session->rtcp.gs.socket,&sa,sizeof(sa))<0){
-			ortp_error("Cannot dissolve connect() association for rtcp socket: %s", getSocketError());
-		}
-		session->flags&=~RTP_SOCKET_CONNECTED;
-		session->flags&=~RTCP_SOCKET_CONNECTED;
+
+		ortp_message("RtpSession [%p] sending to rtp %s rtcp %s %s", session, rtp_printable_addr, rtcp_printable_addr, is_aux ? "as auxiliary destination" : "");
+	} else {
+		ortp_message("RtpSession [%p] sending to rtp %s %s", session, rtp_printable_addr, is_aux ? "as auxiliary destination" : "");
 	}
-	ortp_message("RtpSession [%p] sending to rtp %s rtcp %s %s",session
-									,rtp_printable_addr
-									,rtcp_printable_addr
-									,is_aux ? "as auxiliary destination" : "");
-	end:
+	/*Apply DSCP setting. On windows the destination address is required for doing this.*/
+	rtp_session_set_dscp(session, -1);
+end:
 	if (is_aux){
 		if (err==-1){
 			ortp_free(aux_rtp);
@@ -1051,9 +1057,77 @@ int rtp_session_sendto(RtpSession *session, bool_t is_rtp, mblk_t *m, int flags,
 	return ret;
 }
 
+static const ortp_recv_addr_t * lookup_recv_addr(RtpSession *session, struct sockaddr *from, socklen_t fromlen) {
+	const ortp_recv_addr_t *result = NULL;
+	bctbx_list_t *iterator = session->recv_addr_map;
+	while (iterator != NULL) {
+		ortp_recv_addr_map_t *item = (ortp_recv_addr_map_t *)bctbx_list_get_data(iterator);
+		uint64_t curtime = ortp_get_cur_time_ms();
+		if ((curtime - item->ts) > 2000) {
+			bctbx_list_t *to_remove = iterator;
+			iterator = bctbx_list_next(iterator);
+			session->recv_addr_map = bctbx_list_erase_link(session->recv_addr_map, to_remove);
+		} else {
+			if (memcmp(&item->ss, from, fromlen) == 0) result = &item->recv_addr;
+			iterator = bctbx_list_next(iterator);
+		}
+	}
+	return result;
+}
+
+static const ortp_recv_addr_t * get_recv_addr(RtpSession *session, struct sockaddr *from, socklen_t fromlen) {
+	char result[NI_MAXHOST] = { 0 };
+	char dest[NI_MAXHOST] = { 0 };
+	struct addrinfo *ai = NULL;
+	int port = 0;
+	int family = from->sa_family;
+	int err;
+	err = bctbx_sockaddr_to_ip_address(from, fromlen, dest, sizeof(dest), &port);
+	if (err != 0) {
+		ortp_error("bctbx_sockaddr_to_ip_address failed");
+		return NULL;
+	}
+	err = bctbx_get_local_ip_for(family, dest, port, result, sizeof(result));
+	if (err != 0) {
+		ortp_error("bctbx_get_local_ip_for failed: dest=%s, port=%d", dest, port);
+		return NULL;
+	}
+	ai = bctbx_ip_address_to_addrinfo(family, SOCK_DGRAM, result, port);
+	if (ai == NULL) {
+		ortp_error("bctbx_ip_address_to_addrinfo failed: result=%s, port=%d", result, port);
+		return NULL;
+	} else {
+		ortp_recv_addr_map_t *item = bctbx_new0(ortp_recv_addr_map_t, 1);
+		memcpy(&item->ss, from, fromlen);
+		item->recv_addr.family = family;
+		if (family == AF_INET) {
+			memcpy(&item->recv_addr.addr.ipi_addr, &((struct sockaddr_in *)ai->ai_addr)->sin_addr, sizeof(item->recv_addr.addr.ipi_addr));
+		} else if (family == AF_INET6) {
+			memcpy(&item->recv_addr.addr.ipi6_addr, &((struct sockaddr_in6 *)ai->ai_addr)->sin6_addr, sizeof(item->recv_addr.addr.ipi6_addr));
+		}
+		bctbx_freeaddrinfo(ai);
+		item->ts = ortp_get_cur_time_ms();
+		session->recv_addr_map = bctbx_list_append(session->recv_addr_map, item);
+		return &item->recv_addr;
+	}
+}
+
 int rtp_session_recvfrom(RtpSession *session, bool_t is_rtp, mblk_t *m, int flags, struct sockaddr *from, socklen_t *fromlen) {
 	int ret = rtp_session_rtp_recv_abstract(is_rtp ? session->rtp.gs.socket : session->rtcp.gs.socket, m, flags, from, fromlen);
-	if (ret >= 0) {
+	if ((ret >= 0) && (session->use_pktinfo == TRUE)) {
+		if (m->recv_addr.family == AF_UNSPEC) {
+			/* The receive address has not been filled, this typically happens on Mac OS X when receiving an IPv4 packet on
+			 * a dual stack socket. Try to guess the address because it is mandatory for ICE. */
+			const ortp_recv_addr_t *recv_addr = lookup_recv_addr(session, from, *fromlen);
+			if (recv_addr == NULL) {
+				recv_addr = get_recv_addr(session, from, *fromlen);
+			}
+			if (recv_addr != NULL) {
+				memcpy(&m->recv_addr, recv_addr, sizeof(ortp_recv_addr_t));
+			} else {
+				ortp_error("Did not succeed to fill the receive address, this should not happen! [family=%d, len=%d]", from->sa_family, (int)*fromlen);
+			}
+		}
 		/* Store the local port in the recv_addr of the mblk_t, the address is already filled in rtp_session_rtp_recv_abstract */
 		m->recv_addr.port = htons(is_rtp ? session->rtp.gs.loc_port : session->rtcp.gs.loc_port);
 	}
@@ -1069,19 +1143,22 @@ void update_sent_bytes(OrtpStream *os, int nbytes) {
 	os->sent_bytes += nbytes + overhead;
 }
 
-static void update_recv_bytes(OrtpStream *os, int nbytes) {
+static void update_recv_bytes(OrtpStream *os, size_t nbytes, const struct timeval *recv_time) {
 	int overhead = ortp_stream_is_ipv6(os) ? IP6_UDP_OVERHEAD : IP_UDP_OVERHEAD;
 	if ((os->recv_bytes == 0) && (os->recv_bw_start.tv_sec == 0) && (os->recv_bw_start.tv_usec == 0)) {
 		ortp_gettimeofday(&os->recv_bw_start, NULL);
 	}
-	os->recv_bytes += nbytes + overhead;
+	os->recv_bytes += (unsigned int)(nbytes + overhead);
+	ortp_bw_estimator_packet_received(&os->recv_bw_estimator, nbytes + overhead, recv_time);
 }
 
 static void log_send_error(RtpSession *session, const char *type, mblk_t *m, struct sockaddr *destaddr, socklen_t destlen){
 	char printable_ip_address[65]={0};
+	int errnum = getSocketErrorCode();
+	const char *errstr = getSocketError();
 	bctbx_sockaddr_to_printable_ip_address(destaddr, destlen, printable_ip_address, sizeof(printable_ip_address));
-	ortp_error ("RtpSession [%p] error sending [%s] packet [%p] to %s: %s",
-		session, type, m, printable_ip_address, getSocketError());
+	ortp_error ("RtpSession [%p] error sending [%s] packet [%p] to %s: %s [%d]",
+		session, type, m, printable_ip_address, errstr, errnum);
 }
 
 static int rtp_session_rtp_sendto(RtpSession * session, mblk_t * m, struct sockaddr *destaddr, socklen_t destlen, bool_t is_aux){
@@ -1259,8 +1336,8 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 		ret = bytes_received;
 #endif
 		for (cmsghdr = CMSG_FIRSTHDR(&msghdr); cmsghdr != NULL ; cmsghdr = CMSG_NXTHDR(&msghdr, cmsghdr)) {
-#if defined(ORTP_TIMESTAMP)
-			if (cmsghdr->cmsg_level == SOL_SOCKET && cmsghdr->cmsg_type == SO_TIMESTAMP) {
+#ifdef SO_TIMESTAMP
+			if (cmsghdr->cmsg_level == SOL_SOCKET && cmsghdr->cmsg_type == SCM_TIMESTAMP) {
 				memcpy(&msg->timestamp, (struct timeval *)CMSG_DATA(cmsghdr), sizeof(struct timeval));
 			}
 #endif
@@ -1305,9 +1382,20 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 			}
 #endif
 		}
+
+		if (msg->timestamp.tv_sec == 0){
+			static int warn_once = 1; /*VERY BAD to use a static but there is no context in this function to hold this variable*/
+			if (warn_once){
+				ortp_warning("This platform doesn't implement SO_TIMESTAMP, will use gettimeofday() instead.");
+				warn_once = 0;
+			}
+			ortp_gettimeofday(&msg->timestamp, NULL);
+		}
 		/*store recv addr for use by modifiers*/
-		memcpy(&msg->net_addr,from,*fromlen);
-		msg->net_addrlen = *fromlen;
+		if (from && fromlen) {
+			memcpy(&msg->net_addr,from,*fromlen);
+			msg->net_addrlen = *fromlen;
+		}
 	}
 	return ret;
 }
@@ -1365,24 +1453,12 @@ static void compute_rtcp_xr_statistics(RtpSession *session, mblk_t *block, const
 	}
 }
 
-static void notify_tmmbr_received(RtpSession *session, mblk_t *m) {
-	if (session->eventqs != NULL) {
-		rtcp_fb_tmmbr_fci_t *fci = rtcp_RTPFB_tmmbr_get_fci(m);
-		OrtpEvent *ev = ortp_event_new(ORTP_EVENT_TMMBR_RECEIVED);
-		OrtpEventData *d = ortp_event_get_data(ev);
-		d->packet = copymsg(m);
-		d->info.tmmbr_mxtbr = rtcp_fb_tmmbr_fci_get_mxtbr_mantissa(fci) * (1 << rtcp_fb_tmmbr_fci_get_mxtbr_exp(fci));
-		rtp_session_dispatch_event(session, ev);
-	}
-}
-
 static void handle_rtcp_rtpfb_packet(RtpSession *session, mblk_t *block) {
 	switch (rtcp_RTPFB_get_type(block)) {
 		case RTCP_RTPFB_TMMBR:
 			if (session->rtcp.tmmbr_info.received) freemsg(session->rtcp.tmmbr_info.received);
 			session->rtcp.tmmbr_info.received = copymsg(block);
 			rtp_session_send_rtcp_fb_tmmbn(session, rtcp_RTPFB_get_packet_sender_ssrc(block));
-			notify_tmmbr_received(session, block);
 			break;
 		case RTCP_RTPFB_TMMBN:
 			if (session->rtcp.tmmbr_info.sent) {
@@ -1403,8 +1479,9 @@ static void handle_rtcp_rtpfb_packet(RtpSession *session, mblk_t *block) {
  * @brief : for SR packets, retrieves their timestamp, gets the date, and stores these information into the session descriptor. The date values may be used for setting some fields of the report block of the next RTCP packet to be sent.
  * @param session : the current session descriptor.
  * @param block : the block descriptor that may contain a SR RTCP message.
- * @return -1 if we detect that the packet is in fact a STUN packet, otherwise 0.
+ * @return 0 if the packet is a real RTCP packet, -1 otherwise. 
  * @note a basic parsing is done on the block structure. However, if it fails, no error is returned, and the session descriptor is left as is, so it does not induce any change in the caller procedure behaviour.
+ * @note the packet is freed or is taken ownership if -1 is returned
  */
 static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct sockaddr *addr, socklen_t addrlen ) {
 	rtcp_common_header_t *rtcp;
@@ -1412,8 +1489,9 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 
 	int msgsize = (int) ( block->b_wptr - block->b_rptr );
 	if ( msgsize < RTCP_COMMON_HEADER_SIZE ) {
-		ortp_debug( "Receiving a too short RTCP packet" );
-		return 0;
+		ortp_warning( "Receiving a too short RTCP packet" );
+		freemsg(block);
+		return -1;
 	}
 
 	rtcp = (rtcp_common_header_t *)block->b_rptr;
@@ -1424,6 +1502,8 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 		stunlen = ntohs(stunlen);
 		if (stunlen + 20 == block->b_wptr - block->b_rptr) {
 			/* this looks like a stun packet */
+			rtp_session_update_remote_sock_addr(session, block, FALSE, TRUE);
+			
 			if (session->eventqs != NULL) {
 				OrtpEvent *ev = ortp_event_new(ORTP_EVENT_STUN_PACKET_RECEIVED);
 				OrtpEventData *ed = ortp_event_get_data(ev);
@@ -1432,15 +1512,16 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 				memcpy(&ed->source_addr,addr,addrlen);
 				ed->info.socket_type = OrtpRTCPSocket;
 				rtp_session_dispatch_event(session, ev);
+				return -1;
 			}
 		}else{
-			/* discard in two case: the packet is not stun OR nobody is interested by STUN (no eventqs) */
 			ortp_warning("RtpSession [%p] receiving rtcp packet with version number != 2, discarded", session);
 		}
+		freemsg(block);
 		return -1;
 	}
 
-	update_recv_bytes(&session->rtcp.gs, (int)(block->b_wptr - block->b_rptr));
+	update_recv_bytes(&session->rtcp.gs, (int)(block->b_wptr - block->b_rptr), &block->timestamp);
 
 	/* compound rtcp packet can be composed by more than one rtcp message */
 	do{
@@ -1486,6 +1567,8 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 		}
 	}while (rtcp_next_packet(block));
 	rtcp_rewind(block);
+	
+	rtp_session_update_remote_sock_addr(session, block, FALSE, FALSE);
 	return 0;
 }
 
@@ -1501,10 +1584,6 @@ static void rtp_process_incoming_packet(RtpSession * session, mblk_t * mp, bool_
 
 	struct sockaddr *remaddr = NULL;
 	socklen_t addrlen;
-
-	if (!mp){
-		return;
-	}
 	remaddr = (struct sockaddr *)&mp->net_addr;
 	addrlen = mp->net_addrlen;
 
@@ -1514,10 +1593,13 @@ static void rtp_process_incoming_packet(RtpSession * session, mblk_t * mp, bool_
 		rtp_session_do_splice(session, mp, is_rtp_packet);
 	}
 
-	/* store the sender RTP address to do symmetric RTP at start mainly for stun packets.
-	 * --For rtp packet symmetric RTP is handled in rtp_session_rtp_parse() after first valid rtp packet received.
-	 * --For rtcp, only swicth if valid rtcp packet && first rtcp packet received*/
-	rtp_session_update_remote_sock_addr(session,mp,is_rtp_packet,is_rtp_packet || (rtp_get_version(mp) != 2));
+	/*
+	 * Symmetric RTP policy
+	 * - if a STUN packet is received AND it is the first packet received, switch destination.
+	 * - if a RTP or RTCP packet is received, switch destination.
+	 * In all other cases, we don't switch.
+	 * This logic is implemented in rtp_session_rtp_parse() and process_rtcp_packet().
+	**/
 
 	if (is_rtp_packet){
 		if (session->use_connect && session->symmetric_rtp && !sock_connected ){
@@ -1527,7 +1609,7 @@ static void rtp_process_incoming_packet(RtpSession * session, mblk_t * mp, bool_
 			}
 		}
 		/* then parse the message and put on jitter buffer queue */
-		update_recv_bytes(&session->rtp.gs, (int)(mp->b_wptr - mp->b_rptr));
+		update_recv_bytes(&session->rtp.gs, (size_t)(mp->b_wptr - mp->b_rptr), &mp->timestamp);
 		rtp_session_rtp_parse(session, mp, user_ts, remaddr,addrlen);
 		/*for bandwidth measurements:*/
 	}else {
@@ -1536,7 +1618,7 @@ static void rtp_process_incoming_packet(RtpSession * session, mblk_t * mp, bool_
 					session->flags|=RTCP_SOCKET_CONNECTED;
 			}
 		}
-		if (process_rtcp_packet(session, mp, remaddr, addrlen) >= 0){
+		if (process_rtcp_packet(session, mp, remaddr, addrlen) == 0){
 			/* a copy is needed since rtp_session_notify_inc_rtcp will free the mp,
 			and we don't want to send RTCP XR packet before notifying the application
 			that a message has been received*/
@@ -1557,7 +1639,7 @@ void rtp_session_process_incoming(RtpSession * session, mblk_t *mp, bool_t is_rt
 	if (session->net_sim_ctx && session->net_sim_ctx->params.mode == OrtpNetworkSimulatorInbound) {
 		/*drain possible packets queued in the network simulator*/
 		mp = rtp_session_network_simulate(session, mp, &is_rtp_packet);
-		rtp_process_incoming_packet(session, mp, is_rtp_packet, ts, received_via_rtcp_mux); /*BUG here: received_via_rtcp_mux is not preserved by network simulator*/
+		if (mp) rtp_process_incoming_packet(session, mp, is_rtp_packet, ts, received_via_rtcp_mux); /*BUG here: received_via_rtcp_mux is not preserved by network simulator*/
 	} else if (mp != NULL) {
 		rtp_process_incoming_packet(session, mp, is_rtp_packet, ts, received_via_rtcp_mux);
 	}
@@ -1597,7 +1679,7 @@ int rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts) {
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
 				}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
-#ifdef __ios
+#if TARGET_OS_IPHONE
 				/*hack for iOS and non-working socket because of background mode*/
 				if (errnum==ENOTCONN){
 					/*re-create new sockets */
@@ -1660,7 +1742,7 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTCP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
 				}else ortp_warning("Error receiving RTCP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
-#ifdef __ios
+#if TARGET_OS_IPHONE
 				/*hack for iOS and non-working socket because of background mode*/
 				if (errnum==ENOTCONN){
 					/*re-create new sockets */
@@ -1682,7 +1764,7 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 
 int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool_t is_rtp,bool_t only_at_start) {
 	struct sockaddr_storage * rem_addr = NULL;
-	socklen_t rem_addrlen;
+	socklen_t *rem_addrlen;
 	const char* socket_type;
 	bool_t sock_connected;
 	bool_t do_address_change = /*(rtp_get_version(mp) == 2 && */ !only_at_start;
@@ -1692,13 +1774,13 @@ int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool
 
 	if (is_rtp) {
 		rem_addr = &session->rtp.gs.rem_addr;
-		rem_addrlen = session->rtp.gs.rem_addrlen;
+		rem_addrlen = &session->rtp.gs.rem_addrlen;
 		socket_type = "rtp";
 		sock_connected = session->flags & RTP_SOCKET_CONNECTED;
 		do_address_change =  session->rtp.gs.socket != (ortp_socket_t)-1  && ( do_address_change || rtp_session_get_stats(session)->packet_recv == 0);
 	} else {
 		rem_addr = &session->rtcp.gs.rem_addr;
-		rem_addrlen = session->rtcp.gs.rem_addrlen;
+		rem_addrlen = &session->rtcp.gs.rem_addrlen;
 		sock_connected = session->flags & RTCP_SOCKET_CONNECTED;
 		socket_type = "rtcp";
 		do_address_change = session->rtcp.gs.socket != (ortp_socket_t)-1  && (do_address_change || rtp_session_get_stats(session)->recv_rtcp_packets == 0);
@@ -1712,7 +1794,7 @@ int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool
 		char current_ip_address[64]={0};
 		char new_ip_address[64]={0};
 
-		bctbx_sockaddr_to_printable_ip_address((struct sockaddr *)rem_addr, rem_addrlen, current_ip_address, sizeof(current_ip_address));
+		bctbx_sockaddr_to_printable_ip_address((struct sockaddr *)rem_addr, *rem_addrlen, current_ip_address, sizeof(current_ip_address));
 		bctbx_sockaddr_to_printable_ip_address((struct sockaddr *)&mp->net_addr, mp->net_addrlen, new_ip_address, sizeof(new_ip_address));
 		ortp_message("Switching %s destination from %s to %s for session [%p]"
 			   , socket_type
@@ -1721,7 +1803,14 @@ int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool
 			   , session);
 
 		memcpy(rem_addr,&mp->net_addr,mp->net_addrlen);
+		*rem_addrlen = mp->net_addrlen;
+#ifdef WIN32
+		if (is_rtp){
+			/*re-apply dscp settings for the new destination (windows specific).*/
+			rtp_session_set_dscp( session, -1 );
+		}
+#endif
 		return 0;
 	}
 	return -1;
-	}
+}
