@@ -1,27 +1,28 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc1889) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
+ * Copyright (C) 2017 Belledonne Communications SARL
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
 
 #include "ortp/str_utils.h"
+#include "utils.h"
 
 
 void qinit(queue_t *q){
@@ -39,6 +40,8 @@ void mblk_init(mblk_t *mp)
 void mblk_meta_copy(const mblk_t *source, mblk_t *dest) {
 	dest->reserved1 = source->reserved1;
 	dest->reserved2 = source->reserved2;
+	memcpy(&dest->net_addr,&source->net_addr,source->net_addrlen);
+	dest->net_addrlen = source->net_addrlen;
 #if defined(ORTP_TIMESTAMP)
 	dest->timestamp = source->timestamp;
 #endif
@@ -56,19 +59,38 @@ dblk_t *datab_alloc(size_t size){
 	return db;
 }
 
-static ORTP_INLINE void datab_ref(dblk_t *d){
+void dblk_ref(dblk_t *d){
+#if HAVE_STDATOMIC_H
+	atomic_fetch_add_explicit(&d->db_ref, 1, memory_order_relaxed);
+#else
 	d->db_ref++;
+#endif
 }
 
-static ORTP_INLINE void datab_unref(dblk_t *d){
-	d->db_ref--;
-	if (d->db_ref==0){
+void dblk_unref(dblk_t *d){
+#if HAVE_STDATOMIC_H
+	atomic_int previous_ref = atomic_fetch_sub_explicit(&d->db_ref, 1, memory_order_release);
+#else
+	int previous_ref = d->db_ref--;
+#endif
+	if (previous_ref == 1){
 		if (d->db_freefn!=NULL)
 			d->db_freefn(d->db_base);
 		ortp_free(d);
 	}
 }
 
+unsigned char * dblk_base(dblk_t *db) {
+	return db->db_base;
+}
+
+unsigned char * dblk_lim(dblk_t *db) {
+	return db->db_lim;
+}
+
+int dblk_ref_value(dblk_t *db) {
+	return (int)db->db_ref;
+}
 
 mblk_t *allocb(size_t size, int pri)
 {
@@ -109,7 +131,7 @@ void freeb(mblk_t *mp)
 	return_if_fail(mp->b_datap!=NULL);
 	return_if_fail(mp->b_datap->db_base!=NULL);
 
-	datab_unref(mp->b_datap);
+	dblk_unref(mp->b_datap);
 	ortp_free(mp);
 }
 
@@ -131,7 +153,7 @@ mblk_t *dupb(mblk_t *mp)
 	return_val_if_fail(mp->b_datap!=NULL,NULL);
 	return_val_if_fail(mp->b_datap->db_base!=NULL,NULL);
 
-	datab_ref(mp->b_datap);
+	dblk_ref(mp->b_datap);
 	newm=(mblk_t *) ortp_malloc0(sizeof(mblk_t));
 	mblk_meta_copy(mp, newm);
 	newm->b_datap=mp->b_datap;
@@ -233,9 +255,9 @@ void msgpullup(mblk_t *mp,size_t len)
 	dblk_t *db;
 	size_t wlen=0;
 
-	if (mp->b_cont==NULL && len==-1) return;	/*nothing to do, message is not fragmented */
+	if (mp->b_cont==NULL && len==(size_t)-1) return;	/*nothing to do, message is not fragmented */
 
-	if (len==-1) len=msgdsize(mp);
+	if (len==(size_t)-1) len=msgdsize(mp);
 	db=datab_alloc(len);
 	while(wlen<len && mp!=NULL){
 		int remain=(int)(len-wlen);
@@ -252,7 +274,7 @@ void msgpullup(mblk_t *mp,size_t len)
 	/*set firstm to point to the new datab */
 	freemsg(firstm->b_cont);
 	firstm->b_cont=NULL;
-	datab_unref(firstm->b_datap);
+	dblk_unref(firstm->b_datap);
 	firstm->b_datap=db;
 	firstm->b_rptr=db->db_base;
 	firstm->b_wptr=firstm->b_rptr+wlen;
@@ -341,3 +363,19 @@ mblk_t *msgb_allocator_alloc(msgb_allocator_t *a, size_t size){
 void msgb_allocator_uninit(msgb_allocator_t *a){
 	flushq(&a->q,-1);
 }
+
+void ortp_recvaddr_to_sockaddr(ortp_recv_addr_t *recvaddr, struct sockaddr *addr, socklen_t *socklen) {
+	if (recvaddr->family == AF_INET) {
+		struct sockaddr_in *addr_in = (struct sockaddr_in *)addr;
+		addr_in->sin_family = AF_INET;
+		addr_in->sin_addr = recvaddr->addr.ipi_addr;
+		addr_in->sin_port = recvaddr->port;
+		*socklen = sizeof(struct sockaddr_in);
+	} else if (recvaddr->family == AF_INET6) {
+		struct sockaddr_in6 *addr_in6 = (struct sockaddr_in6 *)addr;
+		addr_in6->sin6_family = AF_INET6;
+		addr_in6->sin6_addr = recvaddr->addr.ipi6_addr;
+		addr_in6->sin6_port = recvaddr->port;
+		*socklen = sizeof(struct sockaddr_in6);
+	}
+}
