@@ -103,11 +103,7 @@ int main(int argc, char *argv[])
 		session[i]=rtp_session_new(RTP_SESSION_RECVONLY);
 		rtp_session_set_scheduling_mode(session[i],1);
 		rtp_session_set_blocking_mode(session[i],0);
-#ifdef ORTP_INET6
 		rtp_session_set_local_addr(session[i],"::",port,port+1);
-#else
-		rtp_session_set_local_addr(session[i],"0.0.0.0",port,port+1);
-#endif
 		rtp_session_set_payload_type(session[i],0);
 		rtp_session_set_recv_buf_size(session[i],256);
 		port+=2;
